@@ -1,7 +1,7 @@
 #
 # randomOnLines.R
 #
-# $Revision: 1.4 $  $Date: 2010/08/14 03:34:17 $
+# $Revision: 1.5 $  $Date: 2012/06/06 09:58:28 $
 #
 # Generate random points on specified lines
 #
@@ -11,8 +11,18 @@ runifpointOnLines <- function(n, L) {
     stop("n should be a single nonnegative integer")
   if(!is.psp(L))
     L <- as.psp(L)
-  if(n == 0)
-    return(ppp(numeric(0), numeric(0), window=as.owin(L)))
+  X <- datagen.runifpointOnLines(n, L)
+  out <- ppp(X$x, X$y, window=as.owin(L))
+  return(out)
+}
+
+datagen.runifpointOnLines <- function(n, L) {
+  stopifnot(is.psp(L))
+  if(n == 0) 
+    return(data.frame(x=numeric(0),
+                      y=numeric(0),
+                      seg=integer(0),
+                      tp=numeric(0)))
   len <- lengths.psp(L)
   cumlen <- cumsum(len)
   cum0len <- c(0, cumlen)
@@ -23,15 +33,14 @@ runifpointOnLines <- function(n, L) {
   # parametric position along segment
   tt <- (uu - cum0len[kk])/len[kk]
   tt[!is.finite(tt)] <- 0
-  # map to (x,y)
+  # convert to (x,y)
   Ldf <- as.data.frame(L)
   dx <- with(Ldf, x1-x0)
   dy <- with(Ldf, y1-y0)
   x <- with(Ldf, x0[kk] + tt * dx[kk])
   y <- with(Ldf, y0[kk] + tt * dy[kk])
-  # return
-  out <- ppp(x, y, window=as.owin(L))
-  return(out)
+  #
+  return(data.frame(x=x, y=y, seg=kk, tp=tt))
 }
 
 runifpoisppOnLines <- function(lambda, L) {
@@ -40,19 +49,33 @@ runifpoisppOnLines <- function(lambda, L) {
     stop("lambda should be a single, finite, nonnegative number")
   if(!is.psp(L))
     L <- as.psp(L)
+  X <- datagen.runifpoisppOnLines(lambda, L)
+  out <- ppp(X$x, X$y, window=as.owin(L))
+  return(out)
+}
+
+datagen.runifpoisppOnLines <- function(lambda, L) {
+  stopifnot(is.psp(L))
   mu <- lambda * sum(lengths.psp(L))
   n <- rpois(1, mu)
-  out <- runifpointOnLines(n, L)
-  return(out)
+  df <- datagen.runifpointOnLines(n, L)
+  return(df)
 }
 
 rpoisppOnLines <- function(lambda, L, lmax=NULL, ...) {
   if(!(is.numeric(lambda) || is.function(lambda) || is.im(lambda)))
     stop(paste(sQuote("lambda"),
                "must be a constant, a function or an image"))
-  if(is.numeric(lambda)) 
-    return(runifpoisppOnLines(lambda, L))
+  if(!is.psp(L))
+    L <- as.psp(L)
+  X <- datagen.rpoisppOnLines(lambda, L, lmax=lmax, ...)
+  out <- ppp(X$x, X$y, window=as.owin(L))
+}
 
+datagen.rpoisppOnLines <- function(lambda, L, lmax=NULL, ...)  {
+  stopifnot(is.psp(L))
+  if(is.numeric(lambda)) 
+    return(datagen.runifpoisppOnLines(lambda, L))
   if(is.im(lambda)) {
     if(!(lambda$type %in% c("real", "integer")))
         stop("lambda must be numeric-valued or integer-valued")
@@ -73,22 +96,21 @@ rpoisppOnLines <- function(lambda, L, lmax=NULL, ...) {
     if(!is.finite(lmax))
       stop("Infinite values of lambda obtained")
   } 
-
-  # rejection method
-  Y <- runifpoisppOnLines(lmax, L)
-  n <- Y$n
+  # Lewis-Shedler (rejection) method
+  Y <- datagen.runifpoisppOnLines(lmax, L)
+  n <- nrow(Y)
   if(n == 0)
     return(Y)
   # evaluate lambda at each simulated point
   if(is.function(lambda)) 
     lambdaY <- lambda(Y$x, Y$y, ...)
   else
-    lambdaY <- safelookup(lambda, Y)
+    lambdaY <- safelookup(lambda, as.ppp(Y, W=as.owin(L)))
   lambdaY[is.na(lambdaY)] <- 0
   # accept/reject
   pY <- lambdaY/lmax
   retain <- (runif(n) < pY)
-  Y <- Y[retain]
+  Y <- Y[retain, , drop=FALSE]
   return(Y)
 }
 
